use std::{cell::RefCell, rc::Rc};

use crate::__COMPILED_BLUEPRINT_MAP__;
use gtk4::{
    glib,
    prelude::{ApplicationExt, GtkWindowExt},
    Application,
};
use gtk_blueprint::get_blp;
use libadwaita::prelude::{PreferencesGroupExt, PreferencesRowExt};
use tracing::trace;

use crate::protocol::BackToFront;

pub(crate) fn get_app(
    from_backend_rx: Rc<RefCell<Option<tokio::sync::mpsc::Receiver<BackToFront>>>>,
) -> Application {
    let app = Application::builder()
        .application_id("de.teilgedanken.sieverman")
        .build();

    app.connect_activate(move |app| build_ui(app, from_backend_rx.clone()));

    app
}

fn build_ui(
    app: &gtk4::Application,
    from_backend_rx: Rc<RefCell<Option<tokio::sync::mpsc::Receiver<BackToFront>>>>,
) {
    let builder = gtk4::Builder::new();
    builder
        .add_from_string(get_blp!("gui/main_window.blp"))
        .expect("Failed to parse blueprint");
    let main_window = builder
        .object::<libadwaita::ApplicationWindow>("window")
        .unwrap();
    let prefrences_group = builder
        .object::<libadwaita::PreferencesGroup>("server_settings")
        .unwrap();
    main_window.set_application(Some(app));

    let future = {
        let mut data_event_receiver = from_backend_rx.take().expect("data_event_reciver");
        async move {
            while let Some(event) = data_event_receiver.recv().await {
                match event {
                    BackToFront::ServerConnected(caps) => {
                        prefrences_group.add(&row_in_settings("Greeting", &caps.implementation));
                        prefrences_group
                            .add(&row_in_settings("Authentication", &caps.sasl.join(",")));
                        prefrences_group
                            .add(&row_in_settings("Starttls", &caps.starttls.to_string()));
                        prefrences_group.add(&row_in_settings("Version", &caps.version));
                    }
                }
            }
        }
    };

    let c = glib::MainContext::default();
    c.spawn_local(future);
    main_window.present();
    trace!("Window is visible");
}

fn row_in_settings(name: &str, value: &str) -> libadwaita::ActionRow {
    let builder = gtk4::Builder::new();
    builder
        .add_from_string(get_blp!("gui/server_info_row.blp"))
        .expect("Failed to parse blueprint");

    let row = builder
        .object::<libadwaita::ActionRow>("server_info_row")
        .unwrap();
    row.set_title(name);
    let label = builder
        .object::<gtk4::Label>("server_info_row_value")
        .unwrap();
    label.set_label(value);
    row
}
