mod backend;
mod gui;
pub mod protocol;
use std::{cell::RefCell, rc::Rc, thread, time::Duration};

use gtk4::{glib, prelude::ApplicationExtManual as _};
use tracing::{info, trace};

gtk_blueprint::gen_blp_map!("gui");

fn main() -> glib::ExitCode {
    console_subscriber::ConsoleLayer::builder()
        // set how long the console will retain data from completed tasks
        .retention(Duration::from_secs(60))
        // set the address the server is bound to
        .server_addr(([127, 0, 0, 1], 6669))
        .init();

    gtk4::init().expect("Failed to initialize GTK");
    libadwaita::init().expect("Adwaita initialization failed");

    let (to_frontent_tx, from_backend_rx) = tokio::sync::mpsc::channel(5);
    let _handle = thread::spawn(move || {
        info!("Running Backend");
        tokio::runtime::Builder::new_current_thread()
            .enable_io()
            .enable_time()
            .build()
            .unwrap()
            .block_on(backend::run::run(to_frontent_tx))
            .expect("Failed to run or interrupted");
    });

    let app = gui::main_window::get_app(Rc::new(RefCell::new(Some(from_backend_rx))));

    let res = app.run();
    trace!("End");

    res
}
